# skybox.mod

param nPhoto integer, > 0;
set Rel within {1..nPhoto, 1..nPhoto};
param alpha > 0;
param k integer, > 0;
param pts{1..k, 1..2};
param Hpts{Rel, 1..k, 1..2};
param Rp{1..nPhoto, 1..3};

var R{1..nPhoto, 1..3};
var ang{1..nPhoto};
var axis{1..nPhoto, 1..3};
var cs{1..nPhoto};
var sn{1..nPhoto};
var Rm{1..nPhoto, 1..3, 1..3};
var Rrel{Rel, 1..3, 1..3};
var HRptsx{Rel, 1..k};
var HRptsy{Rel, 1..k};
var HRptsz{Rel, 1..k};
var dx{Rel, 1..k};
var dy{Rel, 1..k};

minimize error: sum{(i, j) in Rel, ik in 1..k} sqrt(dx[i,j,ik] * dx[i,j,ik] + dy[i,j,ik] * dy[i,j,ik]);

subject to angle{p in 1..nPhoto}: ang[p] = sqrt(R[p,1] * R[p,1] + R[p,2] * R[p,2] + R[p,3] * R[p,3]);
subject to axisVec{p in 1..nPhoto, i in 1..3}: axis[p,i] = R[p,i] / ang[p];
subject to cosinus{p in 1..nPhoto}: cs[p] = cos(ang[p]);
subject to sinus{p in 1..nPhoto}: sn[p] = sin(ang[p]);
subject to rotation11{p in 1..nPhoto}: Rm[p,1,1] = cs[p] + (1 - cs[p]) * axis[p,1] * axis[p,1];
subject to rotation22{p in 1..nPhoto}: Rm[p,2,2] = cs[p] + (1 - cs[p]) * axis[p,2] * axis[p,2];
subject to rotation33{p in 1..nPhoto}: Rm[p,3,3] = cs[p] + (1 - cs[p]) * axis[p,3] * axis[p,3];
subject to rotation12{p in 1..nPhoto}: Rm[p,1,2] = (1 - cs[p]) * axis[p,1] * axis[p,2] - sn[p] * axis[p,3];
subject to rotation23{p in 1..nPhoto}: Rm[p,2,3] = (1 - cs[p]) * axis[p,2] * axis[p,3] - sn[p] * axis[p,1];
subject to rotation31{p in 1..nPhoto}: Rm[p,3,1] = (1 - cs[p]) * axis[p,3] * axis[p,1] - sn[p] * axis[p,2];
subject to rotation13{p in 1..nPhoto}: Rm[p,1,3] = (1 - cs[p]) * axis[p,1] * axis[p,3] + sn[p] * axis[p,2];
subject to rotation21{p in 1..nPhoto}: Rm[p,2,1] = (1 - cs[p]) * axis[p,2] * axis[p,1] + sn[p] * axis[p,3];
subject to rotation32{p in 1..nPhoto}: Rm[p,3,2] = (1 - cs[p]) * axis[p,3] * axis[p,2] + sn[p] * axis[p,1];
subject to relative_rotation{(a,b) in Rel, i in 1..3, j in 1..3}: Rrel[a,b,i,j] = Rm[b,1,i] * Rm[a,1,j] + Rm[b,1,i] * Rm[a,1,j] + Rm[b,1,i] * Rm[a,1,j];
subject to homogene_pointsx{(a,b) in Rel, ik in 1..k}: HRptsx[a,b,ik] = Rrel[a,b,1,1] * pts[ik,1] + Rrel[a,b,1,2] * pts[ik,2] + Rrel[a,b,1,3] * alpha;
subject to homogene_pointsy{(a,b) in Rel, ik in 1..k}: HRptsy[a,b,ik] = Rrel[a,b,2,1] * pts[ik,1] + Rrel[a,b,2,2] * pts[ik,2] + Rrel[a,b,2,3] * alpha;
subject to homogene_pointsz{(a,b) in Rel, ik in 1..k}: HRptsz[a,b,ik] = Rrel[a,b,3,1] * pts[ik,1] / alpha + Rrel[a,b,3,2] * pts[ik,2] / alpha + Rrel[a,b,3,3];
subject to diffx{(a,b) in Rel, ik in 1..k}: dx[a,b,ik] = HRptsx[a,b,ik] / HRptsz[a,b,ik] - Hpts[a,b,ik,1];
subject to diffy{(a,b) in Rel, ik in 1..k}: dy[a,b,ik] = HRptsy[a,b,ik] / HRptsz[a,b,ik] - Hpts[a,b,ik,2];
